/*
 * Copyright 2018 NXP
 * All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#include "board.h"
#include "fsl_common.h"
#include "fsl_power.h"
#include "fsl_gpio.h"
#include "pin_mux.h"
#include "clock_config.h"
/*******************************************************************************
 * Definitions
 ******************************************************************************/
#define DEMO_WAKEUP_PIN_NUMBER 0U
#define DEMO_WAKE_UP_PIN_IRQ PIO0_IRQn
#define DEMO_LED_GPIO_1 1U
#define DEMO_LED_GPIO_4 4U
#define DEMO_LED_GPIO_5 5U
#define DEMO_LED_GPIO_6 6U

static uint32_t s_currentPowerMode = 0U;
static bool s_powerModeSelected = false;
/*******************************************************************************
 * Prototypes
 ******************************************************************************/
void BOARD_InitLedDisplay(void);
void BOARD_LedDisplay(uint32_t num);
void BOARD_IOCON_SetPin(uint32_t pin, uint32_t func);
void BOARD_EnableDeepSleepIRQ(void);
static void DEMO_GetUserSelection(void);
static void DEMO_WaitUserSelectKey(void);
static void DEMO_LedShow(uint8_t num, uint32_t duration, bool forceInit);

/*******************************************************************************
 * Code
 ******************************************************************************/

void BOARD_IOCON_SetPin(uint32_t pin, uint32_t func)
{
    IOCON->PIO[0][pin] = func;
}

void BOARD_EnableDeepSleepIRQ(void)
{
    SYSCON->STARTAPRP0 |= 1 << DEMO_WAKEUP_PIN_NUMBER;
    SYSCON->STARTERP0 |= 1 << DEMO_WAKEUP_PIN_NUMBER;
}
void PIO0_IRQHandler(void)
{
    /* Change state of switch. */
    if ((GPIO_PortGetInterruptsFlags(GPIO) & (1 << DEMO_WAKEUP_PIN_NUMBER)) &&
        (GPIO_PortReadMaskedInterrupts(GPIO) & (1 << DEMO_WAKEUP_PIN_NUMBER)))
    {
        s_powerModeSelected = true;
        GPIO_PortDisableInterrupts(GPIO, 1 << DEMO_WAKEUP_PIN_NUMBER);
    }
    /* clear the interrupt status */
    GPIO_PortClearInterruptFlags(GPIO, 1U << DEMO_WAKEUP_PIN_NUMBER);
}

void DEMO_PreEnterLowPower(void)
{
    switch (s_currentPowerMode)
    {
        case kPmu_Sleep:
            BOARD_IOCON_SetPin(DEMO_WAKEUP_PIN_NUMBER, IOCON_PIO_FUNC(0));
            GPIO_PortEnableInterrupts(GPIO, 1U << DEMO_WAKEUP_PIN_NUMBER);
            break;
        case kPmu_Deep_Sleep: /* Enter deep sleep mode. */
            BOARD_EnableDeepSleepIRQ();
            BOARD_IOCON_SetPin(DEMO_WAKEUP_PIN_NUMBER, IOCON_PIO_FUNC(0));
            GPIO_PortEnableInterrupts(GPIO, 1U << DEMO_WAKEUP_PIN_NUMBER);
            break;
        case kPmu_Deep_PowerDown: /* Enter deep power down mode. */
            BOARD_IOCON_SetPin(DEMO_WAKEUP_PIN_NUMBER, IOCON_PIO_FUNC(1));
            POWER_EnableWakeupPinForDeepPowerDown(kPmu_Dpd_WakeupPin_PIO0_0, true);
            break;
        default:
            break;
    }
}

void DEMO_LowPowerWakeup(void)
{
    s_powerModeSelected = false;
}

void DEMO_InitWakeupPin(void)
{
    gpio_pin_config_t config = {
        .direction = kGPIO_DigitalInput, .outputLogic = 0U,
    };
    GPIO_PortInit(GPIO, DEMO_WAKEUP_PIN_NUMBER);
    GPIO_PinInit(GPIO, DEMO_WAKEUP_PIN_NUMBER, &config);
    GPIO_PinSetInterruptConfig(GPIO, DEMO_WAKEUP_PIN_NUMBER, kGPIO_IntRisingEdge);
}

/*!
 * @brief Main function
 */
int main(void)
{
    /* Init board hardware. */
    BOARD_InitPins();
    BOARD_InitBootClocks();
    CLOCK_SetSysClkDiv(kCLOCK_SysClkFreq4MHZ);
    SystemCoreClockUpdate();

    DEMO_InitWakeupPin();

    while (1)
    {
        DEMO_GetUserSelection();

        /* prepare to enter low power mode */
        DEMO_PreEnterLowPower();

        /* Enter the low power mode. */
        switch (s_currentPowerMode)
        {
            case kPmu_Sleep: /* Enter sleep mode. */
                POWER_EnterSleep();
                break;
            case kPmu_Deep_Sleep: /* Enter deep sleep mode. */
                POWER_EnterDeepSleep();
                break;
            case kPmu_Deep_PowerDown: /* Enter deep power down mode. */
                POWER_EnterDeepPowerDownMode();
                break;
            default:
                break;
        }

        /* restore the active mode configurations */
        DEMO_LowPowerWakeup();
    }
}

/*
 * Get user selection from UART.
 */
static void DEMO_GetUserSelection(void)
{
    uint32_t dynamicPowerMode = 0U;

    while (!s_powerModeSelected)
    {
        DEMO_LedShow(dynamicPowerMode, 0x300, true);
        /* wait user select */
        DEMO_WaitUserSelectKey();

        if (s_powerModeSelected)
        {
            break;
        }

        if (++dynamicPowerMode == 3U)
        {
            dynamicPowerMode = 0U;
        }
    }

    DEMO_LedShow(dynamicPowerMode, 0xe0, true);
    s_currentPowerMode = dynamicPowerMode;
}

static void DEMO_LedShow(uint8_t num, uint32_t duration, bool forceInit)
{
    /* Disable GPIO pin interrupt */
    GPIO_PortDisableInterrupts(GPIO, 1U << DEMO_WAKEUP_PIN_NUMBER);
    /* show number */
    BOARD_LedMatrixShowNumber(num, duration, forceInit);
    /* disable LED martrix immediately */
    GPIO_PortClear(GPIO, 2047U);
    /* Enable GPIO pin interrupt */
    GPIO_PortClearInterruptFlags(GPIO, 0xFFFU);
}

static void DEMO_WaitUserSelectKey(void)
{
    volatile uint32_t systemClockFreq_kHz = SystemCoreClock * 3;

    /* Enable GPIO pin interrupt */
    GPIO_PortClearInterruptFlags(GPIO, 0xFFFU);
    NVIC_ClearPendingIRQ(DEMO_WAKE_UP_PIN_IRQ);
    GPIO_PortEnableInterrupts(GPIO, 1U << DEMO_WAKEUP_PIN_NUMBER);
    EnableIRQ(DEMO_WAKE_UP_PIN_IRQ);
    s_powerModeSelected = false;

    while ((systemClockFreq_kHz--) && (!s_powerModeSelected))
    {
        __NOP();
    }
}
