/*
 * Copyright (c) 2018, NXP
 * All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */
#include "board.h"
#include "fsl_iap.h"
#include "pin_mux.h"
#include "clock_config.h"
/*******************************************************************************
 * Definitions
 ******************************************************************************/
#define EEPROM_SOURCE_CLOCK kCLOCK_SysClk
#define EEPROM_CLK_FREQ CLOCK_GetFreq(kCLOCK_SysClk)
#define EXAMPLE_EEPROM EEPROM
#define startSector 10
#define endSector 11
/*******************************************************************************
 * Prototypes
 ******************************************************************************/
void error_trap(void);
/*******************************************************************************
 * Variables
 ******************************************************************************/
uint32_t page_buf[FSL_FEATURE_SYSCON_FLASH_PAGE_SIZE_BYTES / sizeof(uint32_t)];

/*******************************************************************************
 * Code
 ******************************************************************************/

/*
* @brief Gets called when an error occurs.
*
* @details show error message and trap forever.
*/

void error_trap(void)
{
    while (1)
    {
        BOARD_LedMatrixShowNumber(1, 30, 1); /*! failed show number 1 */
    }
}

/*
* @brief Gets called when the app is complete.
*
* @details show finshed message and trap forever.
*/
void app_finalize(void)
{
    while (1)
    {
        BOARD_LedMatrixShowNumber(0, 30, 1); /*! end show number 0*/
    }
}

/*!
* @brief Main function
*/
int main(void)
{
    uint32_t i, status;
    /* init the board hardware */
    BOARD_InitPins();
    BOARD_InitBootClocks();
    CLOCK_SetSysClkDiv(kCLOCK_SysClkFreq4MHZ); // 4M

    /* prepare the theth sector */
    status = IAP_PrepareSectorForWrite(startSector, endSector);
    if (status == kStatus_IAP_Success)
    {
        BOARD_LedMatrixShowNumber(0, 30, 1); /*! Successfully show number 0*/
    }
    else
        error_trap();

    status = IAP_EraseSector(startSector, endSector, SystemCoreClock);
    if (status == kStatus_IAP_Success)
    {
        BOARD_LedMatrixShowNumber(0, 30, 1); /*! Successfully show number 0*/
    }
    else
        error_trap();

    /* Generate data to be written to flash */
    for (i = 0; i < FSL_FEATURE_SYSCON_FLASH_PAGE_SIZE_BYTES; i++)
    {
        *(((uint8_t *)(&page_buf[0])) + i) = i;
    }
    /* Program the tenth sector */
    for (i = 0; i < (FSL_FEATURE_SYSCON_FLASH_SECTOR_SIZE_BYTES / FSL_FEATURE_SYSCON_FLASH_PAGE_SIZE_BYTES); i++)
    {
        IAP_PrepareSectorForWrite(startSector, endSector);
        status = IAP_CopyRamToFlash(
            (startSector * FSL_FEATURE_SYSCON_FLASH_SECTOR_SIZE_BYTES) + (i * FSL_FEATURE_SYSCON_FLASH_PAGE_SIZE_BYTES),
            &page_buf[0], FSL_FEATURE_SYSCON_FLASH_PAGE_SIZE_BYTES, SystemCoreClock);
    }
    /* Verify sector contents */
    for (i = 0; i < (FSL_FEATURE_SYSCON_FLASH_SECTOR_SIZE_BYTES / FSL_FEATURE_SYSCON_FLASH_PAGE_SIZE_BYTES); i++)
    {
        status = IAP_Compare(
            (startSector * FSL_FEATURE_SYSCON_FLASH_SECTOR_SIZE_BYTES) + (i * FSL_FEATURE_SYSCON_FLASH_PAGE_SIZE_BYTES),
            &page_buf[0], FSL_FEATURE_SYSCON_FLASH_PAGE_SIZE_BYTES);

        if (status == kStatus_IAP_Success)
        {
            BOARD_LedMatrixShowNumber(0, 30, 1); /*! Successfully show number 0*/
        }
        else
            error_trap();
    }

    /* nErasing flash sector */
    IAP_PrepareSectorForWrite(startSector, endSector);
    IAP_EraseSector(startSector, endSector, SystemCoreClock);
    status = IAP_BlankCheckSector(startSector, endSector);
    if (status == kStatus_IAP_Success)
    {
        BOARD_LedMatrixShowNumber(0, 30, 1); /*! Successfully show number 0*/
    }
    else
        error_trap();

    /* First write a page into flash sector 10*/
    IAP_PrepareSectorForWrite(startSector, endSector);
    IAP_CopyRamToFlash((startSector * FSL_FEATURE_SYSCON_FLASH_SECTOR_SIZE_BYTES), &page_buf[0],
                       FSL_FEATURE_SYSCON_FLASH_PAGE_SIZE_BYTES, SystemCoreClock);
    /* Erase page */
    IAP_PrepareSectorForWrite(startSector, endSector);
    IAP_ErasePage(
        (startSector * FSL_FEATURE_SYSCON_FLASH_SECTOR_SIZE_BYTES) / FSL_FEATURE_SYSCON_FLASH_PAGE_SIZE_BYTES,
        ((startSector * FSL_FEATURE_SYSCON_FLASH_SECTOR_SIZE_BYTES) / FSL_FEATURE_SYSCON_FLASH_PAGE_SIZE_BYTES) + 1,
        SystemCoreClock);

    /* Fill page buffer with erased state value */
    for (i = 0; i < FSL_FEATURE_SYSCON_FLASH_PAGE_SIZE_BYTES; i++)
    {
        *(((uint8_t *)(&page_buf[0])) + i) = 0xFF;
    }

    /* Verify Erase */
    status = IAP_Compare((startSector * FSL_FEATURE_SYSCON_FLASH_SECTOR_SIZE_BYTES), (uint32_t *)(&page_buf[0]),
                         FSL_FEATURE_SYSCON_FLASH_PAGE_SIZE_BYTES);
    if (status == kStatus_IAP_Success)
    {
        BOARD_LedMatrixShowNumber(0, 30, 1); /*! Successfully show number 0*/
    }
    else
        error_trap();

    /* End of FLASHIAP Example */
    app_finalize();
}
